// Rouslan Synth 2011-2015
#include "midi.h"
#include "synth.h"
#include "lcd.h"
#include "ui.h"

#if SIMULATOR
u8 PORTC;
#endif

u8 red_led_life;
u8 green_led_life;

inline void leds_update()
{
	if(red_led_life) {
		red_led_life--;
		if(red_led_life == 0)
			PORTC &= ~LED_RED;
	}

	if(green_led_life) {
		green_led_life--;
		if(green_led_life == 0)
			PORTC &= ~LED_GREEN;
	}	
}

u8 ctrl_cnt;
void control()
{
		menu_update();
		midi_update();
		synth_update();
		leds_update();

		ctrl_cnt++;	
}

#if SIMULATOR
void synth_main()
#else
//#include <util/delay.h>
__attribute__((naked))
int main(void)
#endif
{
#if !SIMULATOR
	// Configure ports
	DDRB = 0b00000110; // OC1AB are L/R PWM output
	PORTB = 0b00001001; // Activate pull-ups for the switches

	DDRC = 0b11011111; // Port C 2 and 3 are LED output, 5 is slider input (ADC)
	PORTC = 0b00;

	DDRD = 0xFE;	// PORTD is LCD out and USART in

	// Set clock to 8 MHz
	//CLKPR = 0x80; // Enable prescaler change
	//CLKPR = 0x80; // Scaling of 1 (20 MHz)

	// Set timer interrupt for sampling thread
	TCCR0A = 0b10; // Clear Timer on Compare (i.e. when TCNT0 == OCR0A)
	TCCR0B = 0b10; // Increment TCNT0 at CPU_SPEED/8 (= 2.5 MHz)
	OCR0A =  CPU_SPEED / 8 / SAMPLE_RATE;
	TIMSK0 = 0b10;

	// Set timer interrupt for control thread
	TCCR1A = 0b10100001; // non-inverting fast PWM on OC1AB
	TCCR1B = 0b00001001; // Increment TCNT2 at CPU_SPEED (20MHz / 256 = 78.125 KHz max sampling rate)
	//OCR1A = 0x80;
	//OCR1B = 0x80;

	// Set sleep mode
	SMCR = 0b0001; // Stop CPU clock (Ch. 10.9)
	PRR = 0b11010100; // Keep only timer 0, 1, ADC and USART on

	// Slider ADC
	ADMUX = 0b01100101; // ADC5, AVcc
	ADCSRA = 0b11000111; // ~16kHz sampling rate

	// MIDI IN (USART)
	UCSR0B = 0b10010000; // RX interrupt
	UCSR0C = 0b00000110; // 8bit words
	UBRR0L = 40; // 31.25kHz baud rate

	sei();
#endif
	synth_init();
	lcd_init();
	menu_init();

#if !SIMULATOR

	while(1) {
		synth_sample();
		sleep_cpu();
	}
#endif
}

