// Rouslan Synth 2011-2015
#include "declarations.h"
#if !SIMULATOR
#include <avr/interrupt.h>
#endif
#include "midi.h"
#include "synth.h"
#include "lcd.h"

typedef struct 
{
	u8 status;
	u8 byte1;
	u8 byte2;
} midi_cmd;

midi_cmd cmds[MIDI_CMD_BUFFER];
u8 wr_cmd = 0;
u8 rd_cmd;

u8 midi_debug;

void note_off(channel* c, u8 note)
{
	u8 i;
	u8 last = c->note_idx;
	note -= MIDI_NOTE_OFFSET;

	for(i=0; i<MIDI_CMD_BUFFER; i++) {
		u8 ii = c->note_idx - i & MIDI_CMD_BUFFER - 1;
		if(c->notes[ii] == note)
			c->notes[ii] = 0;
		else if(c->notes[last] == 0 && c->notes[ii] > 0) {
			last = ii;
		}
	}
	if(c->notes[last] > 0) {
		c->note_idx = last;
		compute_ds(c, 0);
	} else {
		c->state = DECAYING;
		c->notes[c->note_idx] = note;
	}
}

void midi_parse()
{
	u8 status	= cmds[rd_cmd].status;
	u8 byte1	= cmds[rd_cmd].byte1;
	u8 byte2	= cmds[rd_cmd].byte2;
	channel* c	= &chan[status & 0xF];
	rd_cmd		= rd_cmd + 1 & MIDI_CMD_BUFFER - 1;

	if(midi_debug) {
		lcd_reprint_all_values = 1;
		lcd_print_values(status, byte1, byte2, 0, 0);
	}

	switch(status >> 4) {
		case MIDI_NOTE_OFF: {
			note_off(c, byte1);
			break;
		}
		case MIDI_NOTE_ON: {
			u8 note;
			if(byte2 == 0) {
				note_off(c, byte1);
				break;
			}

			note = byte1 - MIDI_NOTE_OFFSET;

			if(c->state == PLAYING) {
				c->note_idx = c->note_idx + 1 & MIDI_CMD_BUFFER - 1;
				c->notes[c->note_idx] = note;
				compute_ds(c, 0);
			} else {
				c->vol = byte2;
				synth_env_reset(c->osc[0].env_vol);
				synth_env_reset(c->osc[1].env_vol);
				synth_env_reset(c->env_low_pass);
				c->notes[c->note_idx] = note;
				compute_ds(c, 1);
			}

			if(c->state != PLAYING)
				c->osc[0].cur_sample = c->osc[1].cur_sample = 0;
			c->state = PLAYING;
			break;
		}
		case MIDI_PITCH_BEND: {
			c->bend_modulation = c->bend = (u16)(byte2<<7) + byte1 - 8191;
			compute_ds(c, 1);
			break;
		}
		case MIDI_CONTROL: {
			switch(byte1) {
				case MIDI_CTRL_SLIDER_1:
				case MIDI_CTRL_SLIDER_2:
				case MIDI_CTRL_SLIDER_3:
				case MIDI_CTRL_SLIDER_4:	cur_ctrl = byte1 - MIDI_CTRL_SLIDER_1; synth_update_controls(byte2 << 1); break;
				case MIDI_CTRL_MODULATION:	c->mod = byte2; break;
				case MIDI_CTRL_BREADTH:		c->breadth = byte2; break;
				}
			break;
		}
	}
}

void midi_update()
{
	while(wr_cmd != rd_cmd)
		midi_parse();
}

#if SIMULATOR

void midi_write(u8 status, u8 byte1, u8 byte2)
{
	cmds[wr_cmd].status = status;
	cmds[wr_cmd].byte1	= byte1;
	cmds[wr_cmd].byte2	= byte2;

	wr_cmd = wr_cmd + 1 & MIDI_CMD_BUFFER - 1;

#if MIDI_NEVER_OVERFLOW
	if(wr_cmd == rd_cmd)
		midi_parse();
#endif
}

#else

u8 cur_byte;
u8 ignore_cmd;

ISR(USART_RX_vect) {
	u8 midi_in = UDR0;

	if(midi_in & 0x80) { // Header
		// Listen only to commands for channel 0
		if(midi_in >= (MIDI_STATUS<<4) || (midi_in & 0xF) != 0) {
			ignore_cmd = 1;
			return;
		}
		cur_byte = 0;
		ignore_cmd = 0;
	} else if(ignore_cmd)
		return;

	switch(cur_byte) {
		case 0:
			cmds[wr_cmd].status = midi_in;
			cur_byte++;
			break;
		case 1:
			cmds[wr_cmd].byte1 = midi_in;
			LED_GREEN_ON(1); // Indicate received MIDI in
			cur_byte++;
			break;
		case 2:
		{
			cmds[wr_cmd].byte2 = midi_in;
			u8 status = cmds[wr_cmd].status;
			wr_cmd = wr_cmd + 1 & MIDI_CMD_BUFFER - 1;

			if(rd_cmd == wr_cmd) { // Receive buffer is full
				LED_RED_ON(1);
#if MIDI_NEVER_OVERFLOW
				midi_parse();
#endif
			}

			// Running status
			cmds[wr_cmd].status = status;
			cur_byte = 1;
			break;
		}
	}
}

#endif
